package widebase.stream.codec.rq

import org.jboss.netty.buffer.ChannelBuffer
import org.jboss.netty.channel.Channel
import org.jboss.netty.channel.ChannelHandlerContext
import org.jboss.netty.handler.codec.frame.CorruptedFrameException

import widebase.stream.codec. { MessageTypeDecoder, ResponseDecoder }

/** Decode broker message.
 *
 * @author myst3r10n
 */
class BrokerDecoder extends ResponseDecoder with MessageTypeDecoder {

  import vario.data.sizeOf

  @throws(classOf[Exception])
  override def decode(
    ctx: ChannelHandlerContext,
    channel: Channel,
    buffer: ChannelBuffer): Object = {

    val frame = strip(ctx, channel, buffer)

    if(frame == null)
      return null

    val id = readId(frame)

    if(id.isEmpty)
      return null

    readMessage(id.get, frame)

  }

  @throws(classOf[CorruptedFrameException])
  override protected def readMessage(id: Int, buffer: ChannelBuffer): Object = {

    val messageType =
      try {
  
        MessageType(id)

      } catch {

        case e: NoSuchElementException =>
          buffer.resetReaderIndex
          throw new CorruptedFrameException("Unfamiliar with message id: " + id)

      }

    messageType match {

      case MessageType.EndOfDayMessage => new EndOfDayMessage

      case MessageType.RollbackMessage =>
        if(buffer.readableBytes < sizeOf.int) {

          buffer.resetReaderIndex
          return null

        }

        val records = buffer.readInt

        val partition = readString(buffer)

        if(partition == null) {

          buffer.resetReaderIndex
          null

        }

        new RollbackMessage(records, partition)

      case MessageType.TableMessage =>
        if(buffer.readableBytes < sizeOf.int) {

          buffer.resetReaderIndex
          return null

        }

        val length = buffer.readInt

        if(buffer.readableBytes < length) {

          buffer.resetReaderIndex
          return null

        }

        val decodedBytes = Array.ofDim[Byte](length)

        buffer.readBytes(decodedBytes)

        new TableMessage(decodedBytes)

      case MessageType.UnsubscribeMessage => new UnsubscribeMessage
      case _ => super.readMessage(id, buffer)

    }
  }
}

