package widebase.stream.codec.rq

import org.jboss.netty.buffer. { ChannelBuffer, ChannelBuffers }
import org.jboss.netty.channel.Channel
import org.jboss.netty.channel.ChannelHandlerContext

import widebase.stream.codec. { MessageTypeEncoder, ResponseEncoder}

/** Encode broker message.
 *
 * @author myst3r10n
 */
class BrokerEncoder extends ResponseEncoder with MessageTypeEncoder {

  @throws(classOf[Exception])
  override def encode(
    ctx: ChannelHandlerContext,
    channel: Channel,
    msg: Object): Object = writeMessage(ChannelBuffers.dynamicBuffer, msg)

  override protected def writeMessage(buffer: ChannelBuffer, message: Object): Object = {

    message match {

      case message: EndOfDayMessage => writeId(buffer, MessageType.EndOfDayMessage.id)

      case message: RollbackMessage =>
        writeId(buffer, MessageType.RollbackMessage.id)
        buffer.writeInt(message.records)
        writeString(buffer, message.partition)

      case message: TableMessage =>
        writeId(buffer, MessageType.TableMessage.id)
        buffer.writeInt(message.bytes.length)
        buffer.writeBytes(message.bytes)

      case message: UnsubscribeMessage => writeId(buffer, MessageType.UnsubscribeMessage.id)
      case message => return super.writeMessage(buffer, message)

    }

    finish(buffer)

    buffer

  }
}

