package widebase.stream.codec.rq

import org.jboss.netty.buffer.ChannelBuffer
import org.jboss.netty.channel.Channel
import org.jboss.netty.channel.ChannelHandlerContext

import org.jboss.netty.handler.codec.frame. {

  CorruptedFrameException,
  LengthFieldBasedFrameDecoder

}

import widebase.stream.codec.MessageTypeDecoder

/** Decode request.
 *
 * @author myst3r10n
 */
class RequestDecoder
  extends widebase.stream.codec.RequestDecoder
  with MessageTypeDecoder {

  import MessageType.MessageType

  import vario.data.sizeOf

  @throws(classOf[Exception])
  override def decode(
    ctx: ChannelHandlerContext,
    channel: Channel,
    buffer: ChannelBuffer): Object = {

    val frame = strip(ctx, channel, buffer)

    if(frame == null)
      return null

    val id = readId(frame)

    if(id.isEmpty)
      return null

    readMessage(id.get, frame)

  }

  @throws(classOf[CorruptedFrameException])
  override protected def readMessage(id: Int, buffer: ChannelBuffer): Object = {

    val messageType =
      try {

        MessageType(id)

      } catch {

        case e: NoSuchElementException =>
          buffer.resetReaderIndex
          throw new CorruptedFrameException("Unfamiliar with message id: " + id)

      }

    messageType match {

      case MessageType.PublishMessage =>
        val name = readString(buffer)

        if(name == null) {

          buffer.resetReaderIndex
          null

        }

        if(buffer.readableBytes < sizeOf.int) {

          buffer.resetReaderIndex
          return null

        }

        val length = buffer.readInt

        if(buffer.readableBytes < length) {

          buffer.resetReaderIndex
          return null

        }

        val decodedBytes = Array.ofDim[Byte](length)

        buffer.readBytes(decodedBytes)

        new PublishMessage(name, decodedBytes)

      case MessageType.SubscribeMessage =>
        val name = readString(buffer)

        if(name == null) {

          buffer.resetReaderIndex
          null

        }

        val selector = readString(buffer)

        if(selector == null) {

          buffer.resetReaderIndex
          null

        }

        new SubscribeMessage(name, selector)

      case _ => super.readMessage(id, buffer)

    }
  }
}

